==> metalsmith-1.4.1.tgz: package/lib/index.js <==

var assert = require('assert');
var clone = require('clone');
var front = require('front-matter');
var fs = require('co-fs-extra');
var is = require('is');
var Mode = require('stat-mode');
var path = require('path');
var readdir = require('recursive-readdir');
var rm = require('rimraf');
var thunkify = require('thunkify');
var unyield = require('unyield');
var utf8 = require('is-utf8');
var Ware = require('ware');

var isAbsolute = path.isAbsolute;
var relative = path.relative;
var resolve = path.resolve;

/**
 * Thunks.
 */

readdir = thunkify(readdir);
rm = thunkify(rm);

/**
 * Export `Metalsmith`.
 */

module.exports = Metalsmith;

/**
 * Initialize a new `Metalsmith` builder with a working `directory`.
 *
 * @param {String} directory
 */

function Metalsmith(directory){
  if (!(this instanceof Metalsmith)) return new Metalsmith(directory);
  assert(directory, 'You must pass a working directory path.');
  this.ware = new Ware();
  this.directory(directory);
  this.metadata({});
  this.source('src');
  this.destination('build');
  this.clean(true);
  this.frontmatter(true);
}

/**
 * Add a `plugin` to the middleware stack.
 *
 * @param {Function or Array or Ware} plugin
 * @return {Metalsmith}
 */

Metalsmith.prototype.use = function(plugin){
  this.ware.use(plugin);
  return this;
};

/**
 * Get or set the working `directory`.
 *
 * @param {Object} directory
 * @return {Object or Metalsmith}
 */

Metalsmith.prototype.directory = function(directory){
  if (!arguments.length) return resolve(this._directory);
  assert(is.string(directory), 'You must pass a directory path string.');
  this._directory = directory;
  return this;
};

/**
 * Get or set the global `metadata` to pass to templates.
 *
 * @param {Object} metadata
 * @return {Object or Metalsmith}
 */

Metalsmith.prototype.metadata = function(metadata){
  if (!arguments.length) return this._metadata;
  assert(is.object(metadata), 'You must pass a metadata object.');
  this._metadata = clone(metadata);
  return this;
};

/**
 * Get or set the source directory.
 *
 * @param {String} path
 * @return {String or Metalsmith}
 */

Metalsmith.prototype.source = function(path){
  if (!arguments.length) return this.path(this._source);
  assert(is.string(path), 'You must pass a source path string.');
  this._source = path;
  return this;
};

/**
 * Get or set the destination directory.
 *
 * @param {String} path
 * @return {String or Metalsmith}
 */

Metalsmith.prototype.destination = function(path){
  if (!arguments.length) return this.path(this._destination);
  assert(is.string(path), 'You must pass a destination path string.');
  this._destination = path;
  return this;
};

/**
 * Get or set whether the destination directory will be removed before writing.
 *
 * @param {Boolean} clean
 * @return {Boolean or Metalsmith}
 */
Metalsmith.prototype.clean = function(clean){
  if (!arguments.length) return this._clean;
  assert(is.boolean(clean), 'You must pass a boolean.');
  this._clean = clean;
  return this;
};

/**
 * Optionally turn off frontmatter parsing.
 *
 * @param {Boolean} frontmatter
 * @return {Boolean or Metalsmith}
 */

Metalsmith.prototype.frontmatter = function(frontmatter){
  if (!arguments.length) return this._frontmatter;
  assert(is.boolean(frontmatter), 'You must pass a boolean.');
  this._frontmatter = frontmatter;
  return this;
};

/**
 * Resolve `paths` relative to the root directory.
 *
 * @param {String} paths...
 * @return {String}
 */

Metalsmith.prototype.path = function(){
  var paths = [].slice.call(arguments);
  paths.unshift(this.directory());
  return resolve.apply(path, paths);
};

/**
 * Build with the current settings to the destination directory.
 *
 * @return {Object}
 */

Metalsmith.prototype.build = unyield(function*(){
  var clean = this.clean();
  var dest = this.destination();
  if (clean) yield rm(dest);

  var files = yield this.read();
  files = yield this.run(files);
  yield this.write(files);
  return files;
});

/**
 * Run a set of `files` through the middleware stack.
 *
 * @param {Object} files
 * @return {Object}
 */

Metalsmith.prototype.run = unyield(function*(files){
  var run = thunkify(this.ware.run.bind(this.ware));
  var res = yield run(files, this);
  return res[0];
});

/**
 * Read a dictionary of files from a `dir`, parsing frontmatter. If no directory
 * is provided, it will default to the source directory.
 *
 * @param {String} dir (optional)
 * @return {Object}
 */

Metalsmith.prototype.read = unyield(function*(dir){
  dir = dir || this.source();
  var read = this.readFile.bind(this);
  var paths = yield readdir(dir);
  var files = yield paths.map(read);

  return paths.reduce(function(memo, file, i){
    file = relative(dir, file);
    memo[file] = files[i];
    return memo;
  }, {});
});

/**
 * Read a file by `path`. If the path is not absolute, it will be resolved
 * relative to the source directory.
 *
 * @param {String} path
 * @return {Object}
 */

Metalsmith.prototype.readFile = unyield(function*(path){
  var src = this.source();
  if (!isAbsolute(path)) path = resolve(src, path);

  try {
    var frontmatter = this.frontmatter();
    var stats = yield fs.stat(path);
    var buffer = yield fs.readFile(path);
    var file = {};

    if (frontmatter && utf8(buffer)) {
      try {
        var parsed = front(buffer.toString());
      } catch (e) {
        e = new Error('Invalid frontmatter in the file at: ' + path);
        e.code = 'invalid_frontmatter';
        throw e;
      }

      file = parsed.attributes;
      file.contents = new Buffer(parsed.body);
    } else {
      file.contents = buffer;
    }

    file.mode = Mode(stats).toOctal();
    file.stats = stats;
  } catch (e) {
    if (e.code == 'invalid_frontmatter') throw e;
    e.message = 'Failed to read the file at: ' + path + '\n\n' + e.message;
    e.code = 'failed_read';
    throw e;
  }
  return file;
});

/**
 * Write a dictionary of `files` to a destination `dir`. If no directory is
 * provided, it will default to the destination directory.
 *
 * @param {Object} files
 * @param {String} dir (optional)
 */

Metalsmith.prototype.write = unyield(function*(files, dir){
  dir = dir || this.destination();
  var write = this.writeFile.bind(this);

  yield Object.keys(files).map(function(file){
    var path = resolve(dir, file);
    return write(path, files[file]);
  });
});

/**
 * Write a file by `path` with `data`. If the path is not absolute, it will be
 * resolved relative to the destination directory.
 *
 * @param {String} path
 * @param {Object} data
 */

Metalsmith.prototype.writeFile = unyield(function*(path, data){
  var dest = this.destination();
  if (!isAbsolute(path)) path = resolve(dest, path);

  try {
    yield fs.outputFile(path, data.contents);
    if (data.mode) yield fs.chmod(path, data.mode);
  } catch (e) {
    e.message = 'Failed to write the file at: ' + file + '\n\n' + e.message;
    throw e;
  }
});

==> metalsmith-1.4.2.tgz: package/lib/index.js <==

var assert = require('assert');
var clone = require('clone');
var matter = require('gray-matter');
var fs = require('co-fs-extra');
var is = require('is');
var Mode = require('stat-mode');
var path = require('path');
var readdir = require('recursive-readdir');
var rm = require('rimraf');
var thunkify = require('thunkify');
var unyield = require('unyield');
var utf8 = require('is-utf8');
var Ware = require('ware');

var isAbsolute = path.isAbsolute;
var relative = path.relative;
var resolve = path.resolve;

/**
 * Thunks.
 */

readdir = thunkify(readdir);
rm = thunkify(rm);

/**
 * Export `Metalsmith`.
 */

module.exports = Metalsmith;

/**
 * Initialize a new `Metalsmith` builder with a working `directory`.
 *
 * @param {String} directory
 */

function Metalsmith(directory){
  if (!(this instanceof Metalsmith)) return new Metalsmith(directory);
  assert(directory, 'You must pass a working directory path.');
  this.ware = new Ware();
  this.directory(directory);
  this.metadata({});
  this.source('src');
  this.destination('build');
  this.clean(true);
  this.frontmatter(true);
}

/**
 * Add a `plugin` to the middleware stack.
 *
 * @param {Function or Array or Ware} plugin
 * @return {Metalsmith}
 */

Metalsmith.prototype.use = function(plugin){
  this.ware.use(plugin);
  return this;
};

/**
 * Get or set the working `directory`.
 *
 * @param {Object} directory
 * @return {Object or Metalsmith}
 */

Metalsmith.prototype.directory = function(directory){
  if (!arguments.length) return resolve(this._directory);
  assert(is.string(directory), 'You must pass a directory path string.');
  this._directory = directory;
  return this;
};

/**
 * Get or set the global `metadata` to pass to templates.
 *
 * @param {Object} metadata
 * @return {Object or Metalsmith}
 */

Metalsmith.prototype.metadata = function(metadata){
  if (!arguments.length) return this._metadata;
  assert(is.object(metadata), 'You must pass a metadata object.');
  this._metadata = clone(metadata);
  return this;
};

/**
 * Get or set the source directory.
 *
 * @param {String} path
 * @return {String or Metalsmith}
 */

Metalsmith.prototype.source = function(path){
  if (!arguments.length) return this.path(this._source);
  assert(is.string(path), 'You must pass a source path string.');
  this._source = path;
  return this;
};

/**
 * Get or set the destination directory.
 *
 * @param {String} path
 * @return {String or Metalsmith}
 */

Metalsmith.prototype.destination = function(path){
  if (!arguments.length) return this.path(this._destination);
  assert(is.string(path), 'You must pass a destination path string.');
  this._destination = path;
  return this;
};

/**
 * Get or set whether the destination directory will be removed before writing.
 *
 * @param {Boolean} clean
 * @return {Boolean or Metalsmith}
 */
Metalsmith.prototype.clean = function(clean){
  if (!arguments.length) return this._clean;
  assert(is.boolean(clean), 'You must pass a boolean.');
  this._clean = clean;
  return this;
};

/**
 * Optionally turn off frontmatter parsing.
 *
 * @param {Boolean} frontmatter
 * @return {Boolean or Metalsmith}
 */

Metalsmith.prototype.frontmatter = function(frontmatter){
  if (!arguments.length) return this._frontmatter;
  assert(is.boolean(frontmatter), 'You must pass a boolean.');
  this._frontmatter = frontmatter;
  return this;
};

/**
 * Resolve `paths` relative to the root directory.
 *
 * @param {String} paths...
 * @return {String}
 */

Metalsmith.prototype.path = function(){
  var paths = [].slice.call(arguments);
  paths.unshift(this.directory());
  return resolve.apply(path, paths);
};

/**
 * Build with the current settings to the destination directory.
 *
 * @return {Object}
 */

Metalsmith.prototype.build = unyield(function*(){
  var clean = this.clean();
  var dest = this.destination();
  if (clean) yield rm(dest);

  var files = yield this.read();
  files = yield this.run(files);
  yield this.write(files);
  return files;
});

/**
 * Run a set of `files` through the middleware stack.
 *
 * @param {Object} files
 * @return {Object}
 */

Metalsmith.prototype.run = unyield(function*(files){
  var run = thunkify(this.ware.run.bind(this.ware));
  var res = yield run(files, this);
  return res[0];
});

/**
 * Read a dictionary of files from a `dir`, parsing frontmatter. If no directory
 * is provided, it will default to the source directory.
 *
 * @param {String} dir (optional)
 * @return {Object}
 */

Metalsmith.prototype.read = unyield(function*(dir){
  dir = dir || this.source();
  var read = this.readFile.bind(this);
  var paths = yield readdir(dir);
  var files = yield paths.map(read);

  return paths.reduce(function(memo, file, i){
    file = relative(dir, file);
    memo[file] = files[i];
    return memo;
  }, {});
});

/**
 * Read a file by `path`. If the path is not absolute, it will be resolved
 * relative to the source directory.
 *
 * @param {String} path
 * @return {Object}
 */

Metalsmith.prototype.readFile = unyield(function*(path){
  var src = this.source();
  if (!isAbsolute(path)) path = resolve(src, path);

  try {
    var frontmatter = this.frontmatter();
    var stats = yield fs.stat(path);
    var buffer = yield fs.readFile(path);
    var file = {};

    if (frontmatter && utf8(buffer)) {
      try {
        var parsed = matter(buffer.toString());
      } catch (e) {
        e = new Error('Invalid frontmatter in the file at: ' + path);
        e.code = 'invalid_frontmatter';
        throw e;
      }

      file = parsed.data;
      file.contents = new Buffer(parsed.content);
    } else {
      file.contents = buffer;
    }

    file.mode = Mode(stats).toOctal();
    file.stats = stats;
  } catch (e) {
    if (e.code == 'invalid_frontmatter') throw e;
    e.message = 'Failed to read the file at: ' + path + '\n\n' + e.message;
    e.code = 'failed_read';
    throw e;
  }
  return file;
});

/**
 * Write a dictionary of `files` to a destination `dir`. If no directory is
 * provided, it will default to the destination directory.
 *
 * @param {Object} files
 * @param {String} dir (optional)
 */

Metalsmith.prototype.write = unyield(function*(files, dir){
  dir = dir || this.destination();
  var write = this.writeFile.bind(this);

  yield Object.keys(files).map(function(file){
    var path = resolve(dir, file);
    return write(path, files[file]);
  });
});

/**
 * Write a file by `path` with `data`. If the path is not absolute, it will be
 * resolved relative to the destination directory.
 *
 * @param {String} path
 * @param {Object} data
 */

Metalsmith.prototype.writeFile = unyield(function*(path, data){
  var dest = this.destination();
  if (!isAbsolute(path)) path = resolve(dest, path);

  try {
    yield fs.outputFile(path, data.contents);
    if (data.mode) yield fs.chmod(path, data.mode);
  } catch (e) {
    e.message = 'Failed to write the file at: ' + file + '\n\n' + e.message;
    throw e;
  }
});
